// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: ARF
// Author: Richard Saxton
//
// Description: Provides an interface to ARF data
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: ARF.cc,v 1.1 2013/04/02 17:58:21 aibarra Exp $

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"
#include "EnergySpectrum.h"
#include "ARF.h"
#include "CalOalUtility.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class ARF

// Supports data consisting of an energy, energy width and effective area

// -----------------------------------------------------------------------------------

  /* Constructor - from a fluxed spectrum */
  ARF::ARF(const string & arfName)
    {
      PROC;

      // Read file
      DataSet * set   = dataSetServer->open(arfName, DataSet::Read);
     
      // Open the spectrum table
      Table * table = set->table("SPECRESP");

      // Store the table size
      _N = table->rows();
      cout << "opened dataset "<< arfName << " with " <<_N
           <<" energies"<<endl;

      // Read in the energy_lo, energy_hi and specresp columns
      vector<float> energy_lo, energy_hi;
      CalOalUtility::fillVector(table, "ENERG_LO", energy_lo);
      CalOalUtility::fillVector(table, "ENERG_HI", energy_hi);
      CalOalUtility::fillVector(table, "SPECRESP", _value);

      // Save the central energy, energy_width and value arrays
      for (int i=0; i<_N; i++) {
          _energy.push_back(float((energy_lo[i] + energy_hi[i]) / 2.0));
          _energyWidth.push_back(float(energy_hi[i] - energy_lo[i]));
      }
 
    }

/* Destructor */
ARF:: ~ARF()
{
  PROC;
  ENDPROC;
};


