// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// XMM Science Analysis System
//
// Aitor Ibarra

#ifndef EPICINSTRINFO_H
#define EPICINSTRINFO_H

#include <string>
#include <vector>
#include <map>
#include <Xmm.h>
#include <ParameterInfo.h>


using namespace std;

class	EpicInstrInfo {

public:
	EpicInstrInfo();
	EpicInstrInfo(string obsID, Xmm::Instrument instr,
			string expoID);
	~EpicInstrInfo(){}

	void setObsID(string);
	string getObsID();
	void setInstr(Xmm::Instrument );
	Xmm::Instrument getInstr();
	void setExposure(double );
	double getExposure();
	void setBackscale(double );
	double getBackscale();
	void setExposureID(string expoID);
	string getExposureID();
	void setType(string type);
	string getType();
	void setFileName(string fileName);
	string getFileName();

	void setTLMin(int);
	int getTLMin();

	void setTLMax(int);
	int getTLMax();

	void setNPI(int);
	int getNPI();

	void pushSpectra(EpicInstrInfo *e);
	vector<EpicInstrInfo*> getSpectra(string type);

	double getExposure(int id, string type);

private:
	string _fileName;
	string _obsID;
	Xmm::Instrument _instr;
	string _expoID;
	string _type;
	double _exposure;
	double _backscale;
	int _tlmin;
	int _tlmax;
	int _npi;
	vector<EpicInstrInfo*> _spectra;
};

#endif

