// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 * EpicManip.cc
 *
 *  Created on: Mar 5, 2013
 *      Author: aibarra
 */


#include "EpicManip.h"
#include "param.h"

EpicManip::EpicManip()
{
	_src = 0;
	_bkg = 0;
}

EpicManip::~EpicManip()
{
	if(_src != 0)
		dataSetServer->close(_src);

	if(_bkg != 0)
		dataSetServer->close(_bkg);



}

void EpicManip::addSpectra(vector<EpicInstrInfo*> epicVec)
{



	string fileName;
	if (epicVec[0]->getType() == "SRC" )
	{
			_srcName = fileName = stringParameter("filepha");
	}
	else
	{
			_bkgName = fileName = stringParameter("filebkg");
	}

	DataSet *ori = dataSetServer->clone(epicVec[0]->getFileName(),fileName,Dal::Modify);

	/* Access spectrum  */
	Table * spectrumtable = ori->table("SPECTRUM");
	ColumnData *countsDat = spectrumtable->column("COUNTS")->data();
	int32 *counts = countsDat->int32Data();
	double exposure = epicVec[0]->getExposure();
	double backscale = epicVec[0]->getBackscale();
	for (unsigned int i = 1 ; i < epicVec.size() ; ++i)
	{

		DataSet *set =  dataSetServer->open(epicVec[i]->getFileName(),Dal::Read);
		Table *sTable = set->table("SPECTRUM");
		ColumnData *cDat = sTable->column("COUNTS")->data();
		int32 *c = cDat->int32Data();

		for (unsigned int j = 0 ; j < sTable->rows() ; ++j)
		{
			//cout << "c1 " << j << " " << counts[j] << " " << c[j] << endl;
			counts[j] = counts[j] +  c[j];
		}

		exposure = exposure + epicVec[i]->getExposure();
		backscale = backscale + epicVec[i]->getBackscale();
		delete cDat;
		delete sTable;
		dataSetServer->close(set);

	}

	*spectrumtable->addAttribute("EXPOSURE")= exposure/double(epicVec.size());
	*spectrumtable->addAttribute("BACKSCALE")= backscale;

	delete countsDat;

	if (spectrumtable->hasAttribute("BACKFILE"))
		*spectrumtable->attribute("BACKFILE") = "None";
	if (spectrumtable->hasAttribute("RESPFILE"))
		*spectrumtable->attribute("RESPFILE") = "None";
	if (spectrumtable->hasAttribute("ANCRFILE"))
		*spectrumtable->attribute("ANCRFILE") = "None";

	*spectrumtable->attribute("INSTRUME") = "EPIC";
	*ori->attribute("INSTRUME") = "EPIC";
	//Remove GTI extensions
	errstr << "Removing GTI extensions... "<< Sas::message(Msg::App,Msg::Verbose);
	unsigned int blocks = ori->blocks();
	for (unsigned int i = 0 ; i < blocks ; ++i)
	{
		if ( (ori->block(i)->name().find("GTI") != std::string::npos) ||
				 (ori->block(i)->name().find("STDGTI") != std::string::npos) )
		{
			ori->deleteBlock(i);

			blocks = ori->blocks();
			i = 0;
		}
	}


	if (epicVec[0]->getType() == "SRC" )
		_src = ori;
	else
		_bkg = ori;



}


